use strict;
use Cwd;
use Win32::OLE qw(in with);
use Win32::OLE::Const 'Microsoft Excel';

my $lang           = "ENG";
my $MAX_PSNR_ERROR = 0.001;

my %excelWording = (
	"FR" => {
		FLOAT_SEP => ",",
		ROW_TAG   => "L",
		COL_TAG   => "C",
	},
	"ENG" => {
		FLOAT_SEP => ".",
		ROW_TAG   => "R",
		COL_TAG   => "C",
	}
);

my $errorLog           = "";
my %generatedStreams   = ();
my %rowOfStream        = ();
my %streamUsed         = ();
my %streamSizeAchieved = ();

my $Excel;

my $resultSheet;
my $offsetCol;
my $Book;
my $propBook;

my $checkBook;
my $checkSheet;

my %excelParams = (
	ColsPerRate     => 6,
	ColsFirstRate   => 1,
	ColFrameSize    => 2,
	ColYPSNR        => 3,
	ColUPSNR        => 4,
	ColVPSNR        => 5,
	ColsPerSequence => 31,
	RowStreamSize   => 11,
	ColStreamSize   => 2,
	RowFrame0       => 13
);

my %classPerCS = (
	"CS1" => [ "A", "B", "C", "D" ],
	"CS2" => [ "B", "C", "D", "E" ],
);

my %WidthToClass = (
	"2560" => "A",
	"1920" => "B",
	"832"  => "C",
	"416"  => "D",
	"1280" => "E",
);

my %seqParams = (

	# id => [ name, class, posInClass]
	"S01" => [ "Traffic",         "A", 1 ],
	"S02" => [ "PeopleOnStreet",  "A", 2 ],
	"S03" => [ "Kimono",          "B", 1 ],
	"S04" => [ "ParkScene",       "B", 2 ],
	"S05" => [ "Cactus",          "B", 3 ],
	"S06" => [ "BasketballDrive", "B", 4 ],
	"S07" => [ "BQTerrace",       "B", 5 ],
	"S08" => [ "BasketballDrill", "C", 1 ],
	"S09" => [ "BQMall",          "C", 2 ],
	"S10" => [ "PartyScene",      "C", 3 ],
	"S11" => [ "RaceHorses",      "C", 4 ],
	"S12" => [ "BasketballPass",  "D", 1 ],
	"S13" => [ "BQSquare",        "D", 2 ],
	"S14" => [ "BlowingBubbles",  "D", 3 ],
	"S15" => [ "RaceHorses",      "D", 4 ],
	"S16" => [ "Vidyo1",          "E", 1 ],
	"S17" => [ "Vidyo3",          "E", 2 ],
	"S18" => [ "Vidyo4",          "E", 3 ],
);

my @SequenceSet = (
	[], # not used.
	[
		'A',  'S01', 'Traffic', 30,   150,  2560,
		1600, 2500,  3500,      5000, 8000, 14000
	],
	[
		'A', 'S02', 'People on Street',
		30, 150, 2560, 1600, 2500, 3500, 5000, 8000, 14000
	],
	[ 'B', 'S03', 'Kimono', 24, 240, 1920, 1080, 1000, 1600, 2500, 4000, 6000 ],
	[
		'B',  'S04', 'ParkScene', 24,   240,  1920,
		1080, 1000,  1600,        2500, 4000, 6000
	],
	[
		'B', 'S05', 'Cactus', 50, 500, 1920, 1080, 2000, 3000, 4500, 7000, 10000
	],
	[
		'B', 'S06', 'BasketballDrive', 50, 500, 1920, 1080, 2000, 3000, 4500,
		7000, 10000
	],
	[
		'B',  'S07', 'BQTerrace', 60,   600,  1920,
		1080, 2000,  3000,        4500, 7000, 10000
	],
	[
		'C', 'S08', 'BasketballDrill', 50, 500, 832, 480, 384, 512, 768, 1200,
		2000
	],
	[ 'C', 'S09', 'BQMall',     60, 600, 832, 480, 384, 512, 768, 1200, 2000 ],
	[ 'C', 'S10', 'PartyScene', 50, 500, 832, 480, 384, 512, 768, 1200, 2000 ],
	[ 'C', 'S11', 'RaceHorses', 30, 300, 832, 480, 384, 512, 768, 1200, 2000 ],
	[
		'D', 'S12', 'BasketballPass', 50, 500, 416, 240, 256, 384, 512, 850,
		1500
	],
	[ 'D', 'S13', 'BQSquare', 60, 600, 416, 240, 256, 384, 512, 850, 1500 ],
	[
		'D', 'S14', 'BlowingBubbles', 50, 500, 416, 240, 256, 384, 512, 850,
		1500
	],
	[ 'D', 'S15', 'RaceHorses', 30, 300, 416,  240, 256, 384, 512, 850, 1500 ],
	[ 'E', 'S16', 'Vidyo1',     60, 600, 1280, 720, 256, 384, 512, 850, 1500 ],
	[ 'E', 'S17', 'Vidyo3',     60, 600, 1280, 720, 256, 384, 512, 850, 1500 ],
	[ 'E', 'S18', 'Vidyo4',     60, 600, 1280, 720, 256, 384, 512, 850, 1500 ],
);

sub SetAutoCalculationMode() {
	$Book->{Application}->{Calculation}    = -4105;
	$Book->{Application}->{ScreenUpdating} = 1;
}

sub SetManualCalculationMode() {
	$Book->{Application}->{Calculation}    = -4135;
	$Book->{Application}->{ScreenUpdating} = 0;
}

sub CheckPSNR() {
	my $tagId     = shift;
	my $frameId   = shift;
	my $component = shift;
	my $prop      = shift;
	my $computed  = shift;

	$prop     =~ s/,/\./;
	$computed =~ s/,/\./;

	if ( abs( $prop - $computed ) > $MAX_PSNR_ERROR ) {
		$errorLog .=
"$tagId, frame $frameId: wrong $component PSNR! $prop vs. $computed\n";
	}

}

sub ParsePSNRFile() {
	my $cs         = shift;
	my $seq        = shift;
	my $ratePoint  = shift;
	my $traceFile  = shift;
	my $streamFile = shift;

	#-- Get file size
	die "Can't find stream $streamFile!" unless -e $streamFile;
	my $streamSize = -s $streamFile;

	#-- parsing log file to get datas
	print "parsing trace: $traceFile\n";
	die "Can't find PNSR file $traceFile!" unless -e $traceFile;

	# datas to be read
	my ( $iFrame, $rTotalBits, $cTotalBits, $rYPSNR, $rUPSNR, $rVPSNR, $rRate );
	my %fRate      = ();
	my %fYPSNR     = ();
	my %fUPSNR     = ();
	my %fVPSNR     = ();
	my $maxFrameId = -1;
	$rTotalBits = -1;
	$rRate      = -1;
	$rYPSNR     = $rUPSNR = $rVPSNR = -1;
	$cTotalBits = -1;

	#--- parse file
	if ( -e $traceFile ) {

		open( FILE, $traceFile ) or die "Can't open file $traceFile!\n";
		my @lines = <FILE>;
		close FILE;
		my $lines = join "", @lines;

		my $s;
		$maxFrameId = -1;
		$cTotalBits = 0;
		foreach $s (@lines) {
			if ( $s =~ /^(\d+)/ ) {
				$iFrame = $1 + 0;
				if ( $iFrame > $maxFrameId ) { $maxFrameId = $iFrame; }
				if ( !defined $fRate{$iFrame} ) {
					$fRate{$iFrame} = 0;
				}
				my @fields = split /[\s\/\(\)]+/, $s;
				$fYPSNR{$iFrame} = $fields[1];
				$fUPSNR{$iFrame} = $fields[2];
				$fVPSNR{$iFrame} = $fields[3];
				$cTotalBits += $fRate{$iFrame};
			}
		}

	}

	my $iSeq = $seq;
	$iSeq =~ s/S//i;
	# $iSeq -= 1; # -1 no longer required.
	my $cid = $cs;
	$cid =~ s/CS/C/;
	my $tag = $seq . "R$ratePoint" . $cid;

	my $maxStreamSize =
	  ( 1000 * $SequenceSet[$iSeq][ 6 + $ratePoint ] * $SequenceSet[$iSeq][4] )
	  / ( 8 * $SequenceSet[$iSeq][3] );
	printf "  Stream Size: $streamSize (max= $maxStreamSize)\n";
	# BUGFIX - R1
	if ( $streamSize >  $maxStreamSize ) {
		$errorLog .= "$tag: overRate: $streamSize / $maxStreamSize\n";
	}
	printf "  maxFrameId $maxFrameId\n";
	if ( $maxFrameId != $SequenceSet[$iSeq][4] - 1 ) {
		$errorLog .= "$tag: Wrong number of frames\n";
	}

	if ( defined $Book ) {

		#-- export results in excel sheet
		my $resultSheetName = "$cs - Class $seqParams{$seq}[1]";
		print "sheet name for $seq: '$resultSheetName'\n";
		$resultSheet = $Book->Worksheets($resultSheetName);

		my $propSheet = $propBook->Worksheets($resultSheetName);

		$offsetCol =
		  1 +
		  ( $seqParams{$seq}[2] - 1 ) * $excelParams{'ColsPerSequence'} +
		  ( $ratePoint - 1 ) * $excelParams{'ColsPerRate'};

		#-- stream size
		$resultSheet->Cells( $excelParams{'RowStreamSize'},
			$excelParams{'ColStreamSize'} + $offsetCol )->{'Value'} =
		  $streamSize;

		for ( $iFrame = 0 ; $iFrame <= $maxFrameId ; $iFrame++ ) {
			my $err;

			#-- frame bits
			$fRate{$iFrame} = $propSheet->Cells(
				$excelParams{'RowFrame0'} + $iFrame,
				$excelParams{'ColFrameSize'} + $offsetCol
			)->{'Value'};
			$resultSheet->Cells(
				$excelParams{'RowFrame0'} + $iFrame,
				$excelParams{'ColFrameSize'} + $offsetCol
			)->{'Value'} = $fRate{$iFrame};

			#-- Y PSNR
			# BUGFIX - R1
			$fYPSNR{$iFrame} =~ s/,/\./;
			&CheckPSNR(
				$tag, $iFrame, "Y",
				$propSheet->Cells(
					$excelParams{'RowFrame0'} + $iFrame,
					$excelParams{'ColYPSNR'} + $offsetCol
				  )->{'Value'},
				$fYPSNR{$iFrame}
			);
			$fYPSNR{$iFrame} =~ s/\./$excelWording{$lang}{FLOAT_SEP}/;
			$resultSheet->Cells(
				$excelParams{'RowFrame0'} + $iFrame,
				$excelParams{'ColYPSNR'} + $offsetCol
			)->{'Value'} = $fYPSNR{$iFrame};

			#-- U PSNR
			# BUGFIX - R1
			$fUPSNR{$iFrame} =~ s/,/\./;
			&CheckPSNR(
				$tag, $iFrame, "U",
				$propSheet->Cells(
					$excelParams{'RowFrame0'} + $iFrame,
					$excelParams{'ColUPSNR'} + $offsetCol
				  )->{'Value'},
				$fUPSNR{$iFrame}
			);
			$fUPSNR{$iFrame} =~ s/\./$excelWording{$lang}{FLOAT_SEP}/;
			$resultSheet->Cells(
				$excelParams{'RowFrame0'} + $iFrame,
				$excelParams{'ColUPSNR'} + $offsetCol
			)->{'Value'} = $fUPSNR{$iFrame};

			#-- V PSNR
			# BUGFIX - R1
			$fVPSNR{$iFrame} =~ s/,/\./;
			&CheckPSNR(
				$tag, $iFrame, "V",
				$propSheet->Cells(
					$excelParams{'RowFrame0'} + $iFrame,
					$excelParams{'ColVPSNR'} + $offsetCol
				  )->{'Value'},
				$fVPSNR{$iFrame}
			);
			$fVPSNR{$iFrame} =~ s/\./$excelWording{$lang}{FLOAT_SEP}/;
			$resultSheet->Cells(
				$excelParams{'RowFrame0'} + $iFrame,
				$excelParams{'ColVPSNR'} + $offsetCol
			)->{'Value'} = $fVPSNR{$iFrame};
		}
	}
}

sub RetreiveInfoFromFileList() {
	my $fileList = shift;

	# open xls file list
	my $FileBook  = $Excel->Workbooks->Open($fileList);
	my $FileSheet = $FileBook->Worksheets(1);

	# set calculation mode to manual
	&SetManualCalculationMode();

	for (
		my $i = 2 ;
		( defined $FileSheet->Cells( $i, 1 )->{'Value'} )
		  && ( $FileSheet->Cells( $i, 1 )->{'Value'} > 0 ) ;
		$i++
	  )
	{
		print "Treating: " . $FileSheet->Cells( $i, 5 )->{'Value'} . "...\n";

		my $seqId           = $FileSheet->Cells( $i, 1 )->{'Value'};
		my $rateId          = $FileSheet->Cells( $i, 2 )->{'Value'};
		my $csId            = $FileSheet->Cells( $i, 3 )->{'Value'};
		my $tagId           = $FileSheet->Cells( $i, 5 )->{'Value'};
		my $bitFile         = $FileSheet->Cells( $i, 6 )->{'Value'};
		my $decodedYUVFile  = $FileSheet->Cells( $i, 7 )->{'Value'};
		my $originalYUVFile = $FileSheet->Cells( $i, 8 )->{'Value'};

		my $nbFrames = $SequenceSet[$seqId][4];
		my $width    = $SequenceSet[$seqId][5];
		my $height   = $SequenceSet[$seqId][6];

		#-- compute PSNR
		my $PSNRfile = "temp/$tagId" . "_PSNR.txt";
		my $cmdLinePSNR =
"PSNRStatic.exe $width $height $originalYUVFile $decodedYUVFile > $PSNRfile";

		print "  cmd: $cmdLinePSNR\n";
		system $cmdLinePSNR;

		#-- fill in excel sheet
		$seqId = "0$seqId" if (length("$seqId") < 2);
		print "sequencid is '$seqId'\n";
		&ParsePSNRFile( "CS$csId", "S$seqId", $rateId, $PSNRfile, $bitFile );
	}
}

#=======   MAIN
die
"Usage: CrossCheckTemplate FileList.xls template.xls PropFile.xls CreatedFile.xls \n"
  unless $#ARGV == 3;

my $dir = cwd();

my $fileList = $ARGV[0];
my $xlsTemplateFile;
my $xlsFile;
my $xlsFileProp;

# create temp dir if it does not exist
mkdir "temp" unless -e "temp";

# get already active Excel application or open new
$Excel = Win32::OLE->GetActiveObject('Excel.Application')
  || Win32::OLE->new( 'Excel.Application', 'Quit' );
$Excel->{'Visible'} = 1;

$xlsTemplateFile = $ARGV[1];
$xlsFileProp     = $ARGV[2];
$xlsFile         = $ARGV[3];
$fileList        = "$dir/$fileList";
$xlsFile         = "$dir/$xlsFile";
$xlsFileProp     = "$dir/$xlsFileProp";
$xlsTemplateFile = "$dir/$xlsTemplateFile";

# open xls template file
$Book = $Excel->Workbooks->Open($xlsTemplateFile);

# save it for later output
if ( -e $xlsFile ) {
	unlink $xlsFile;
}    # need to remove it for enabling saveAs..
$Book->SaveAs(
	{ Filename => $xlsFile, ConflictResolution => xlOtherSessionChanges } );

# set calculation mode to manual
&SetManualCalculationMode();

# open proponent Book
$propBook = $Excel->Workbooks->Open($xlsFileProp);

&RetreiveInfoFromFileList($fileList);

print '=' x 40 . "\n" . "==== ERRORS FOUNDS\n" . '=' x 40 . "\n";
print $errorLog;

&SetAutoCalculationMode();
$Book->Save();

print '-' x 20 . "   finished!\n";

